unit CopyDataFromMainFormUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, DBGrids, Menus, StdCtrls, Db, DBTables;

type
  TForm1 = class(TForm)
    DBGrid1: TDBGrid;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open: TMenuItem;
    CopyToOtherApp: TMenuItem;
    Exit: TMenuItem;
    N1: TMenuItem;
    OD: TOpenDialog;
    DummyList: TListBox;
    procedure FormCreate(Sender: TObject);
    procedure ExitClick(Sender: TObject);
    procedure OpenClick(Sender: TObject);
    procedure CopyToOtherAppClick(Sender: TObject);
  private
    TargetHandle: THandle;
    procedure WMAppPlus10(var Msg: TMessage);
      message wm_App + 10;
  public
    { Public declarations }
  end;

  EWin32Error = class(Exception);

var
  Form1: TForm1;

implementation

uses
  CopyDataFromDataModule;

{$R *.DFM}

procedure TForm1.WMAppPlus10(var Msg: TMessage);
begin
  TargetHandle := Msg.WParam;
end;

function LaunchChildApp: THandle;
const
  ChildApp = 'CopyDataTo.Exe';
var
  SI: TStartupInfo;
  PI: TProcessInformation;
begin
  GetStartupInfo(SI);
  //Pass this form's window handle as a command line parameter
  //Tacky but effective
  if not CreateProcess(nil, PChar(ChildApp + ' ' + IntToStr(Form1.Handle)),
     nil, nil, False, 0, nil, nil, SI, PI) then
    raise EWin32Error.Create('Unable to launch ' + ChildApp);
  Result := PI.HProcess;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Session.Open;
  Session.GetAliasParams('DBDEMOS', DummyList.Items);
  OD.InitialDir := DummyList.Items.Values['PATH'];
  WaitForInputIdle(LaunchChildApp, Infinite);
end;

procedure TForm1.ExitClick(Sender: TObject);
begin
  Application.Terminate
end;

procedure TForm1.OpenClick(Sender: TObject);
begin
  if OD.Execute then
    with DM.TblFrom do
    begin
      DisableControls;
      Screen.Cursor := crHourGlass;
      Self.Enabled := False;
      try
        Close;
        DatabaseName := ExtractFilePath(OD.FileName);
        TableName := ExtractFileName(OD.FileName);
        Open
      finally
        Self.Enabled := True;
        Screen.Cursor := crDefault;
        EnableControls
      end;
    end;
  CopyToOtherApp.Enabled := True
end;

{ lParam of WM_COPYDATA message points to...
  PCopyDataStruct = ^TCopyDataStruct;
  TCopyDataStruct = packed record
    dwData: DWORD;    //4 bytes of custom data
    cbData: DWORD;    //size of other data
    lpData: Pointer;  //pointer to other data which
                      //cannot contain pointers
  end; }
procedure TForm1.CopyToOtherAppClick(Sender: TObject);
const
  StartNewGrid = DWord(-1);
var
  CopyData: TCopyDataStruct;
  Data: ShortString;
  OuterLoop, InnerLoop: Integer;
  Pt: TPoint;

  procedure SendData;
  begin
    SendMessage(TargetHandle, wm_CopyData,
                Handle, Integer(@CopyData))
  end;

begin
  with CopyData, DM.TblFrom do
  begin
    //Send a signal to re-initiate string grid
    Pt.X := FieldCount;
    Pt.Y := RecordCount + 1;
    dwData := StartNewGrid;
    cbData := SizeOf(Pt);
    lpData := @Pt;
    SendData;
    //Now send the field names
    for InnerLoop := 0 to FieldCount - 1 do
    begin
      dwData := InnerLoop;
      Data := Fields[InnerLoop].DisplayLabel;
      cbData := Length(Data);
      lpData := @Data[1];
      SendData;
    end;
    //Now send all the field values
    DisableControls;
    First;
    Screen.Cursor := crHourglass;
    try
      for OuterLoop := 1 to RecordCount do
      begin
        for InnerLoop := 0 to FieldCount - 1 do
        begin
          dwData := InnerLoop;
          Data := Fields[InnerLoop].AsString;
          if Fields[InnerLoop] is TGraphicField then
            Data := '(Graphic)';
          cbData := Length(Data);
          lpData := @Data[1];
          SendData;
        end;
        Next;
      end
    finally
      Screen.Cursor := crDefault;
      EnableControls
    end;
  end;
end;

end.
